@@ -54,6 +54,9 @@ class Location(ModelSQL, ModelView):
     left = fields.Integer('Left', required=True, select=True)
     right = fields.Integer('Right', required=True, select=True)
     childs = fields.One2Many("stock.location", "parent", "Children")
+    flat_childs = fields.Boolean(
+        "Flat Children",
+        help="Check to restrict to one level of children.")
     warehouse = fields.Function(fields.Many2One('stock.location', 'Warehouse'),
         'get_warehouse')
     input_location = fields.Many2One(
@@ -121,10 +124,22 @@ class Location(ModelSQL, ModelView):
                     ),
                 'child_of_warehouse': ('Location "%(location)s" must be a '
                     'child of warehouse "%(warehouse)s".'),
+                'inactive_location_with_moves': (
+                    "The location '%(location)s' must be empty "
+                    "to be deactivated."),
                 })
 
-        parent_domain = []
-        childs_domain = []
+        parent_domain = [
+            ['OR',
+                ('parent.flat_childs', '=', False),
+                ('parent', '=', None),
+                ]
+            ]
+        childs_domain = [
+            If(Eval('flat_childs', False),
+                ('childs', '=', None),
+                ()),
+            ]
         childs_mapping = cls._childs_domain()
         for type_, allowed_parents in cls._parent_domain().iteritems():
             parent_domain.append(If(Eval('type') == type_,
@@ -133,7 +148,7 @@ class Location(ModelSQL, ModelView):
                     ('type', 'in', childs_mapping[type_]), ()))
         cls.parent.domain = parent_domain
         cls.childs.domain = childs_domain
-        cls.childs.depends.append('type')
+        cls.childs.depends.extend(['flat_childs', 'type'])
 
     @classmethod
     def _parent_domain(cls):
@@ -170,8 +185,12 @@ class Location(ModelSQL, ModelView):
     def validate(cls, locations):
         super(Location, cls).validate(locations)
         cls.check_recursion(locations)
+        inactives = []
         for location in locations:
             location.check_type_for_moves()
+            if not location.active:
+                inactives.append(location)
+        cls.check_inactive(inactives)
 
     def check_type_for_moves(self):
         """ Check locations with moves have types compatible with moves. """
@@ -191,6 +210,52 @@ class Location(ModelSQL, ModelView):
                 self.raise_user_error(
                     'invalid_type_for_moves', (self.rec_name,))
 
+    @classmethod
+    def check_inactive(cls, locations):
+        "Check inactive location are empty"
+        assert all(not l.active for l in locations)
+        empty = cls.get_empty_locations(locations)
+        non_empty = set(locations) - set(empty)
+        if non_empty:
+            cls.raise_user_error('inactive_location_with_moves', {
+                    'location': next(iter(non_empty)).rec_name,
+                    })
+
+    @classmethod
+    def get_empty_locations(cls, locations=None):
+        pool = Pool()
+        Move = pool.get('stock.move')
+        if locations is None:
+            locations = cls.search([])
+        if not locations:
+            return []
+        location_ids = map(int, locations)
+        # Use root to compute for all companies
+        # and ensures inactive locations are in the query
+        with Transaction().set_user(0), \
+                Transaction().set_context(active_test=False):
+            query = Move.compute_quantities_query(
+                location_ids, with_childs=True)
+            quantities = Move.compute_quantities(
+                query, location_ids, with_childs=True)
+            empty = set(location_ids)
+            for (location_id, product), quantity in quantities.iteritems():
+                if quantity:
+                    empty.discard(location_id)
+            for sub_ids in grouped_slice(list(empty)):
+                sub_ids = list(sub_ids)
+                moves = Move.search([
+                        ('state', 'not in', ['done', 'cancel']),
+                        ['OR',
+                            ('from_location', 'in', sub_ids),
+                            ('to_location', 'in', sub_ids),
+                            ],
+                        ])
+                for move in moves:
+                    for location in [move.from_location, move.to_location]:
+                        empty.discard(location.id)
+        return cls.browse(empty)
+
     @staticmethod
     def default_active():
         return True
@@ -203,6 +268,10 @@ class Location(ModelSQL, ModelView):
     def default_right():
         return 0
 
+    @classmethod
+    def default_flat_childs(cls):
+        return False
+
     @staticmethod
     def default_type():
         return 'storage'
