@@ -10,19 +10,18 @@ from sql.functions import CurrentTimestamp
 from sql.conditionals import Coalesce
 
 from trytond.model import ModelSQL, ModelView, fields
-from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
-    Button
-from trytond.pyson import PYSONEncoder, Eval, Or
+from trytond.wizard import Wizard, StateTransition
+from trytond.pyson import Eval, Or
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import grouped_slice
 
+from trytond.modules.product import TemplateFunction
 from .move import StockMixin
 
 __all__ = ['Template', 'Product',
-    'ProductByLocationStart', 'ProductByLocation',
-    'ProductQuantitiesByWarehouse', 'ProductQuantitiesByWarehouseStart',
-    'OpenProductQuantitiesByWarehouse',
+    'ProductByLocationContext',
+    'ProductQuantitiesByWarehouse', 'ProductQuantitiesByWarehouseContext',
     'RecomputeCostPrice']
 
 
@@ -186,18 +185,7 @@ class Product(StockMixin, object):
 
     @classmethod
     def recompute_cost_price(cls, products):
-        pool = Pool()
-        Template = pool.get('product.template')
-
-        if hasattr(cls, 'cost_price'):
-            digits = cls.cost_price.digits
-            write = cls.write
-            record = lambda p: p
-        else:
-            digits = Template.cost_price.digits
-            write = Template.write
-            record = lambda p: p.template
-
+        digits = cls.cost_price.digits
         costs = defaultdict(list)
         for product in products:
             if product.type == 'service':
@@ -205,7 +193,7 @@ class Product(StockMixin, object):
             cost = getattr(product,
                 'recompute_cost_price_%s' % product.cost_price_method)()
             cost = cost.quantize(Decimal(str(10.0 ** -digits[1])))
-            costs[cost].append(record(product))
+            costs[cost].append(product)
 
         if not costs:
             return
@@ -217,7 +205,7 @@ class Product(StockMixin, object):
 
         # Enforce check access for account_stock*
         with Transaction().set_context(_check_access=True):
-            write(*to_write)
+            cls.write(*to_write)
 
     def recompute_cost_price_fixed(self):
         return self.cost_price
@@ -230,7 +218,7 @@ class Product(StockMixin, object):
 
         context = Transaction().context
 
-        if hasattr(self.__class__, 'cost_price'):
+        if not isinstance(self.__class__.cost_price, TemplateFunction):
             product_clause = ('product', '=', self.id)
         else:
             product_clause = ('product.template', '=', self.template.id)
@@ -266,67 +254,37 @@ class Product(StockMixin, object):
                         move.company.currency, round=False)
                 unit_price = Uom.compute_price(move.uom, unit_price,
                     self.default_uom)
-                if quantity + qty != 0:
+                if quantity + qty > 0 and quantity >= 0:
                     cost_price = (
                         (cost_price * quantity) + (unit_price * qty)
                         ) / (quantity + qty)
+                elif qty > 0:
+                    cost_price = unit_price
             quantity += qty
         return cost_price
 
 
-class ProductByLocationStart(ModelView):
+class ProductByLocationContext(ModelView):
     'Product by Location'
-    __name__ = 'product.by_location.start'
+    __name__ = 'product.by_location.context'
     forecast_date = fields.Date(
         'At Date', help=('Allow to compute expected '
             'stock quantities for this date.\n'
             '* An empty value is an infinite date in the future.\n'
             '* A date in the past will provide historical values.'))
+    stock_date_end = fields.Function(fields.Date('At Date'),
+        'on_change_with_stock_date_end')
 
     @staticmethod
     def default_forecast_date():
         Date = Pool().get('ir.date')
         return Date.today()
 
-
-class ProductByLocation(Wizard):
-    'Product by Location'
-    __name__ = 'product.by_location'
-    start = StateView('product.by_location.start',
-        'stock.product_by_location_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Open', 'open', 'tryton-ok', default=True),
-            ])
-    open = StateAction('stock.act_location_quantity_tree')
-
-    def do_open(self, action):
-        pool = Pool()
-        Product = pool.get('product.product')
-        Lang = pool.get('ir.lang')
-
-        context = {}
-        product_id = Transaction().context['active_id']
-        context['product'] = product_id
-        if self.start.forecast_date:
-            context['stock_date_end'] = self.start.forecast_date
-        else:
-            context['stock_date_end'] = datetime.date.max
-        action['pyson_context'] = PYSONEncoder().encode(context)
-        product = Product(product_id)
-
-        for code in [Transaction().language, 'en_US']:
-            langs = Lang.search([
-                    ('code', '=', code),
-                    ])
-            if langs:
-                break
-        lang, = langs
-        date = Lang.strftime(context['stock_date_end'],
-            lang.code, lang.date)
-
-        action['name'] += ' - %s (%s) @ %s' % (product.rec_name,
-            product.default_uom.rec_name, date)
-        return action, {}
+    @fields.depends('forecast_date')
+    def on_change_with_stock_date_end(self, name=None):
+        if self.forecast_date is None:
+            return datetime.date.max
+        return self.forecast_date
 
 
 class ProductQuantitiesByWarehouse(ModelSQL, ModelView):
@@ -399,9 +357,9 @@ class ProductQuantitiesByWarehouse(ModelSQL, ModelView):
         return dict((l.id, quantities[l.date]) for l in lines)
 
 
-class ProductQuantitiesByWarehouseStart(ModelView):
+class ProductQuantitiesByWarehouseContext(ModelView):
     'Product Quantities By Warehouse'
-    __name__ = 'stock.product_quantities_warehouse.start'
+    __name__ = 'stock.product_quantities_warehouse.context'
     warehouse = fields.Many2One('stock.location', 'Warehouse', required=True,
         domain=[
             ('type', '=', 'warehouse'),
@@ -417,28 +375,6 @@ class ProductQuantitiesByWarehouseStart(ModelView):
             return warehouses[0].id
 
 
-class OpenProductQuantitiesByWarehouse(Wizard):
-    'Product Quantities By Warehouse'
-    __name__ = 'stock.product_quantities_warehouse'
-    start = StateView('stock.product_quantities_warehouse.start',
-        'stock.product_quantities_warehouse_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Open', 'open_', 'tryton-ok', default=True),
-            ])
-    open_ = StateAction('stock.act_product_quantities_warehouse')
-
-    def do_open_(self, action):
-        Date = Pool().get('ir.date')
-        action['pyson_context'] = PYSONEncoder().encode({
-                'product': Transaction().context['active_id'],
-                'warehouse': self.start.warehouse.id,
-                })
-        action['pyson_search_value'] = PYSONEncoder().encode([
-                ('date', '>=', Date.today()),
-                ])
-        return action, {}
-
-
 class RecomputeCostPrice(Wizard):
     'Recompute Cost Price'
     __name__ = 'product.recompute_cost_price'
